import {
  min, 
  max,
  select, 
  selectAll,
  scaleBand,
  scaleLinear,
  scaleOrdinal,
  axisBottom,
  axisLeft,
  transition,
} from 'd3';
import { colors } from './utils';

export const barplot = () => {
  // Globals
  let config = {};
  let data = [];

  // ----------------
  // 		My Graph
  // ----------------
  const my = (selection) => {
    // ------------------
    // 		 Dimensions
    // ------------------
    const width = config.width;
    const height = config.height;
    const margin = config.margin;
    
    const group = data.map(d => d[config.xKey]); // xValues
    const subgroup = config.yKey;
    
    // ----------------------
    // 			Transition
    // ----------------------
    
    function getTransition() {
      return transition().duration(800)
    }
    
    // -----------------------
    // 			Define Scales
    // -----------------------
    
    const xScale = d3.scaleBand()
    	.domain(group)
    		.padding(0.2)
    	.range([margin.left, width - margin.right]);
                
    const yValues = data.map(d => 
			subgroup.map(key => d[key])
    ).flat();
    
    const yScale = d3.scaleLinear()
    	.domain([0, max(yValues) + 50])
    	.range([height - margin.bottom, margin.top]);
    
    const xSubscale = scaleBand()
    	.domain(subgroup)
    		.padding(0.05)
    	.range([0, xScale.bandwidth()]);
    
    const colorScale = scaleOrdinal()
    	.domain(subgroup)
    	.range(colors);
            
    // ------------------
    // 		 Draw Axes
    // ------------------
    
    const xAxis = selection
    	.selectAll('g.x-axis')
    	.data([null])
    	.join('g')
    		.attr('class', 'x-axis')
    		.attr('transform', `translate(${0}, ${height - margin.bottom})`)
    	.transition(getTransition())
    	.call(axisBottom(xScale))

    const yAxis = selection
    	.selectAll('g.y-axis')
    	.data([null])
    	.join('g')
    		.attr('class', 'y-axis')
    		.attr('transform', `translate(${margin.left}, ${0})`)
    	.transition(getTransition())
    	.call(axisLeft(yScale));
    
    // ----------------------
    //  		Draw Chart
    // ----------------------
    const marks = data.map((d, i) => ({
      id: i,
      group: d.label,
      data: subgroup.map((key, i) => ({
        id: 'x' + i,
      	key: key,
        value: d[key],
        color: colorScale(key),
        display: true,
      }))
    }));
    
    createOrUpdateChart(marks);
    
    function createOrUpdateChart(data) {
      const chart = selection
      	.selectAll('rect')
      	.data(data, d => d.id) // Key function
      	.join('g')
      		.attr('transform', d => `translate(${xScale(d.group)}, ${0})`)
      	.selectAll('rect')
      	.data(d => d.data, d => d.id)
      	.join(
        	(enter) => (
          	enter 
          		.append('rect')
          			.attr('x', (d) => xSubscale(d.key))
                .attr('width', (d) => xSubscale.bandwidth())
                .attr('fill', (d) => d.color)
                .attr('y', (d) => yScale(0)) 
                .attr('height', 0)
          			.call((enter) => 
                  enter
                    .transition(getTransition())
                    .attr('y', (d) => yScale(d.value))
                    .attr('height', (d) => height - yScale(d.value) - margin.bottom)
                    .delay((d, i) => i * 100)	
                )
          ),
          (update) => (
          	update
          		.append('rect')
          		  .attr('x', (d) => xSubscale(d.key))
                .attr('width', (d) => xSubscale.bandwidth())
                .attr('fill', (d) => d.color)
                .attr('y', (d) => yScale(0))
                .attr('height', 0)
          			.call((update) =>
                  update
                    .transition(getTransition())
                    .attr('y', (d) => yScale(d.value))
                    .attr('height', (d) => height - yScale(d.value) - margin.bottom)
                    .delay((d, i) => i * 100)
            		)
          ),
          (exit) => 
          	exit
          		.call((exit) => 
                    exit
                    	.transition(getTransition())
                    	.attr('height', 0)
                    	.delay((d, i) => i * 100)
            	)
        )
    }
    
    // -------------------
    // 			Legend
    // -------------------
    const legendKeys = marks[0].data;
    createOrUpdateLegends(legendKeys);
    
    function createOrUpdateLegends(data) {      
      const legendRectSize = 18;
      const legendSpacing = 4;

      const legend = selection
        .selectAll('.legend')
        .data(data)
        .join('g')
        .attr('class', 'legend')
        .attr('transform', (d, i) => {
          let height = legendRectSize + legendSpacing;
          let offset =  height * 4 / 2;
          let horz = -2 * legendRectSize;
          let vert = i * height - offset + 70; 
          return `translate(${width - margin.right - 100}, ${vert})`
        });
      
      legend
        .append('rect')
        .attr('width', legendRectSize)
        .attr('height', legendRectSize)
        .style('fill', d => d.color)
        .style('stroke', d => d.color);
      
      legend
        .append('text')
        .attr('x', legendRectSize + legendSpacing)
        .attr('y', legendRectSize - legendSpacing)
        .text(d => d.key)
        .style('cursor', 'pointer')
        .style('user-select', 'none')
        .on('click', toggleBars);
    }
    
    function toggleBars(e, d) {
  		const isStrikethrough = d3.select(this).classed("strikethrough");
  		d3.select(this).classed("strikethrough", !isStrikethrough);
      
  		const selectedItems = d3.selectAll('.strikethrough').nodes().map(element => element.__data__.key);
      
      if(selectedItems.length) {
        const newMarks = [...marks.map(m => ({...m, data: m.data.filter(dt => !selectedItems.includes(dt.key))}))]
        console.log(newMarks)
        createOrUpdateChart(newMarks);
      } else {
        createOrUpdateChart(marks);
      }
		}

    // -------------------
    // 			Tooltip
    // -------------------
  };
  
  // -------------------------
  // 		Getters & Setters
  // -------------------------
  
  my.config = function (_) {
    return arguments.length
      ? ((config = _), my)
      : config;
  };
  my.data = function (_) {
    return arguments.length
      ? ((data = _), my)
      : data;
  };

  // ---------------------
  //	 Return the graph
  // ---------------------
  return my;
};
